//+------------------------------------------------------------------+
//|                                                     MACD RSI.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+

extern double TakeProfit    = 50;
extern double Lots          = 0.1;
extern double StopLoss      = 50;
extern double TrailingStop  = 0;
extern int    MacdFast      = 5;
extern int    MacdSlow      = 12;
extern int    MacdSignal    = 100;
extern int    RSIPeriod     = 21; 
double sl = 0;
double myPoint;
   double SetPoint() 
   { 
   double mPoint; 
   
   if (Digits < 4) 
      mPoint = 0.01; 
   else 
      mPoint = 0.0001; 
   
   return(mPoint); 
   }
   
   //+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init() 
  {
  myPoint = SetPoint();
         
   return(0);
  }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int start()
  {
   double MacdCurrent, MacdPrevious, SignalCurrent, RsiCurrent;
   double SignalPrevious;
   int cnt, ticket, total;
// initial data checks
// it is important to make sure that the expert works with a normal
// chart and the user did not make any mistakes setting external 
// variables (Lots, StopLoss, TakeProfit, 
// TrailingStop) in our case, we check TakeProfit
// on a chart of less than 100 bars
   if(Bars<100)
     {
      Print("bars less than 100");
      return(0);  
     }
   if(TakeProfit<10)
     {
      Print("TakeProfit less than 10");
      return(0);  // check TakeProfit
     }
// to simplify the coding and speed up access
// data are put into internal variables
   MacdCurrent=iMACD(NULL,0,MacdFast,MacdSlow,MacdSignal,PRICE_CLOSE,MODE_MAIN,0);
   MacdPrevious=iMACD(NULL,0,MacdFast,MacdSlow,MacdSignal,PRICE_CLOSE,MODE_MAIN,1);
   SignalCurrent=iMACD(NULL,0,MacdFast,MacdSlow,MacdSignal,PRICE_CLOSE,MODE_SIGNAL,0);
   SignalPrevious=iMACD(NULL,0,MacdFast,MacdSlow,MacdSignal,PRICE_CLOSE,MODE_SIGNAL,1);
   
   RsiCurrent=iRSI(NULL,0,RSIPeriod,0,0);

   total=OrdersTotal();
   if(total<1) 
     {
      // no opened orders identified
      if(AccountFreeMargin()<(1000*Lots))
        {
         Print("We have no money. Free Margin = ", AccountFreeMargin());
         return(0);  
        }
      // check for long position (BUY) possibility
      if(RsiCurrent>50 && MacdCurrent>SignalCurrent && MacdPrevious<SignalPrevious)
        {
      if (StopLoss>0)
         sl = Ask - StopLoss * myPoint;
         ticket=OrderSend(Symbol(),OP_BUY,Lots,Ask,3,sl,Ask+TakeProfit*myPoint,"macd RSI",16384,0,Green);
         if(ticket>0)
           {
            if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES)) Print("BUY order opened : ",OrderOpenPrice());
           }
         else Print("Error opening BUY order : ",GetLastError()); 
         return(0); 
        }
      // check for short position (SELL) possibility
      if(RsiCurrent<50 && MacdCurrent<SignalCurrent && MacdPrevious>SignalPrevious)
        {
      if (StopLoss>0)
         sl = Bid + StopLoss * myPoint;
         ticket=OrderSend(Symbol(),OP_SELL,Lots,Bid,3,sl,Bid-TakeProfit*myPoint,"macd RSI",16384,0,Red);
         if(ticket>0)
           {
            if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES)) Print("SELL order opened : ",OrderOpenPrice());
           }
         else Print("Error opening SELL order : ",GetLastError()); 
         return(0); 
        }
      return(0);
     }
   // it is important to enter the market correctly, 
   // but it is more important to exit it correctly...   
   for(cnt=0;cnt<total;cnt++)
     {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if(OrderType()<=OP_SELL &&   // check for opened position 
         OrderSymbol()==Symbol())  // check for symbol
        {
         if(OrderType()==OP_BUY)   // long position is opened
           {
            // should it be closed?
            if(RsiCurrent<50 && MacdCurrent<SignalCurrent && MacdPrevious>SignalPrevious)
                {
                 OrderClose(OrderTicket(),OrderLots(),Bid,3,Violet); // close position
                 return(0); // exit
                }
            // check for trailing stop
            if(TrailingStop>0)  
              {                 
               if(Bid-OrderOpenPrice()>myPoint*TrailingStop)
                 {
                  if(OrderStopLoss()<Bid-myPoint*TrailingStop)
                    {
                     OrderModify(OrderTicket(),OrderOpenPrice(),Bid-myPoint*TrailingStop,OrderTakeProfit(),0,Green);
                     return(0);
                    }
                 }
              }
           }
         else // go to short position
           {
            // should it be closed?
            if(RsiCurrent>50 && MacdCurrent>SignalCurrent && MacdPrevious<SignalPrevious )
              {
               OrderClose(OrderTicket(),OrderLots(),Ask,3,Violet); // close position
               return(0); // exit
              }
            // check for trailing stop
            if(TrailingStop>0)  
              {                 
               if((OrderOpenPrice()-Ask)>(myPoint*TrailingStop))
                 {
                  if((OrderStopLoss()>(Ask+myPoint*TrailingStop)) || (OrderStopLoss()==0))
                    {
                     OrderModify(OrderTicket(),OrderOpenPrice(),Ask+myPoint*TrailingStop,OrderTakeProfit(),0,Red);
                     return(0);
                    }
                 }
              }
           }
        }
     }
   return(0);
  }
// the end.